/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barcel Chvez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/
#include <QtGui>
#include "registdialog.h"
#include "regist.h"
#include "registmanager.h"
#include "persistencemanager.h"
#include "types.h"


View::Management::RegistDialog::RegistDialog(Model::Domain::Regist *regist, QWidget *parent)
    : QDialog(parent), _regist(regist)
{
    createWidgets();
    createConnections();
    setWindowTitle(trUtf8("Registro")+"[*]");
    setFixedSize(300, 307);
    loadRegist();

}

void View::Management::RegistDialog::done(int result)
{
   if(result)
        saveRegist();

    QDialog::done(result);
}


void View::Management::RegistDialog::registModified(bool modified)
{
    setWindowModified(modified);
    _saveButton -> setEnabled(isSaveable() && modified);
}

void View::Management::RegistDialog::createWidgets()
{

    QFont font12;
    font12.setPointSize(12);     


    _lectiLabel = new QLabel(trUtf8("&Lectura inicial:"));
    _lectiLineEdit = new QLineEdit;
    _lectiLineEdit -> setValidator(new QRegExpValidator(QRegExp("[1-9][0-9]{0,4}"), this));
    _lectiLineEdit->setAlignment(Qt::AlignCenter);
    _lectiLineEdit->setFont(font12);
    _lectiLabel -> setBuddy(_lectiLineEdit);

    _lectfLabel = new QLabel(trUtf8("&Lectura final:"));
    _lectfLineEdit = new QLineEdit;
    _lectfLineEdit -> setValidator(new QRegExpValidator(QRegExp("[1-9][0-9]{0,4}"), this));
    _lectfLineEdit -> setAlignment(Qt::AlignCenter);
    _lectfLineEdit->setFont(font12);
    _lectfLabel -> setBuddy(_lectfLineEdit);

    _dateLabel= new QLabel(trUtf8("&Fecha:"));
    _dateEdit = new QDateEdit;
    _dateEdit->setDisplayFormat(DATE_FORMAT);
    _dateEdit->setDate(QDate::currentDate());
    _dateEdit->setAlignment(Qt::AlignCenter);
    _dateEdit->setFont(font12);
    _dateLabel->setBuddy(_dateEdit);

    _timeLabel= new QLabel(trUtf8("&Hora:"));
    _timeEdit = new QTimeEdit;
    _timeEdit->setDisplayFormat(TIME_FORMAT);
    _timeEdit->setTime(QTime::currentTime());
    _timeEdit->setAlignment(Qt::AlignCenter);
    _timeEdit->setFont(font12);
    _timeLabel->setBuddy(_timeEdit);

    _kwhLabel = new QLabel(trUtf8("Kw/h:"));
    _kwhLineEdit = new QLineEdit;
    _kwhLineEdit->setReadOnly(true);
    _kwhLineEdit -> setAlignment(Qt::AlignCenter);
    _kwhLineEdit->setFont(font12);
    _kwhLabel->setBuddy(_kwhLineEdit);

    _importLabel = new QLabel(trUtf8("Importe: $ "));
    _importLineEdit = new QLineEdit;
    _importLineEdit->setText("0.00 $");
    _importLineEdit->setReadOnly(true);
    _importLineEdit -> setAlignment(Qt::AlignCenter);
    _importLineEdit->setFont(font12);
    _importLabel->setBuddy(_importLineEdit);

    QGridLayout *registLayout = new QGridLayout;
    registLayout -> addWidget(_lectiLabel, 0, 0, 1, 1);
    registLayout -> addWidget(_lectiLineEdit, 1, 0, 1, 1);

    registLayout -> addWidget(_lectfLabel, 0, 3, 1, 1);
    registLayout -> addWidget(_lectfLineEdit, 1, 3, 1, 1);

    registLayout -> addWidget(_dateLabel, 2, 0, 1, 1);
    registLayout -> addWidget(_dateEdit, 3, 0, 1, 1);

    registLayout -> addWidget(_timeLabel, 2, 3, 1, 1);
    registLayout -> addWidget(_timeEdit, 3, 3, 1, 1);

    registLayout -> addWidget(_kwhLabel, 4, 0, 1, 1);
    registLayout -> addWidget(_kwhLineEdit, 5, 0, 1, 1);

    registLayout -> addWidget(_importLabel, 4, 3, 1, 1);
    registLayout -> addWidget(_importLineEdit, 5, 3, 1, 1);
    

    _saveButton = new QPushButton(trUtf8("&Guardar"));
    _saveButton -> setIcon(QIcon(":/images/save.png"));
    _saveButton -> setDefault(true);
    _saveButton -> setEnabled(false);
    _cancelButton = new QPushButton(trUtf8("&Cancelar"));
    _cancelButton -> setIcon(QIcon(":/images/cancel.png"));

    _newRegistGbox = new QGroupBox(trUtf8("Crear Registro"));
    _newRegistGbox->setLayout(registLayout);

    QHBoxLayout *bottomLayout = new QHBoxLayout;

    bottomLayout -> addStretch();
    bottomLayout -> addWidget(_saveButton);
    bottomLayout -> addWidget(_cancelButton);

    QVBoxLayout *mainLayout = new QVBoxLayout;

    mainLayout -> addWidget(_newRegistGbox);
    mainLayout -> addLayout(bottomLayout);

    setLayout(mainLayout);
}

void View::Management::RegistDialog::createConnections()
{
    connect(_lectiLineEdit, SIGNAL(textChanged(QString)),
                this, SLOT(registModified()));
    connect(_lectfLineEdit, SIGNAL(textChanged(QString)),
                this, SLOT(registModified()));
    connect(_dateEdit, SIGNAL(dateChanged(QDate)),
                this, SLOT(registModified()));
    connect(_timeEdit, SIGNAL(timeChanged(QTime)),
                this, SLOT(registModified()));

    connect(_lectiLineEdit, SIGNAL(textChanged(QString)),
            this, SLOT(on_lectiLabel_textChanged()));
    connect(_lectfLineEdit, SIGNAL(textChanged(QString)),
            this, SLOT(on_lectfLabel_textChanged()));
    connect(_saveButton, SIGNAL(clicked()),
            this, SLOT(accept()));
    connect(_cancelButton, SIGNAL(clicked()),
            this, SLOT(reject()));
}

void View::Management::RegistDialog::loadRegist()
{
    int precisionMoney = MAX_MONEY_PRECISION;

    _dateEdit->setDate(_regist->date());
    _timeEdit->setTime(_regist->time());
    _lectfLineEdit->setText(QString::number(_regist->lectf()));
    _lectiLineEdit->setText(QString::number(_regist->lecti()));
    _kwhLineEdit->setText(QString::number(_regist->kwh()));
    _importLineEdit->setText(QString::number(_regist->import(),'f', precisionMoney));
    registModified(false);

    }

void View::Management::RegistDialog::saveRegist()
{
    int id = Model::Management::RegistManager::getId();
    _regist -> setId(id);
    _regist ->setDate(_dateEdit->date());
    _regist ->setTime(_timeEdit->time());
    _regist ->setLectI(_lectiLineEdit->text().toInt());
    _regist ->setLectF(_lectfLineEdit->text().toInt());
    _regist ->setKwh(_kwhLineEdit->text().toInt());
    _regist ->setImport(_importLineEdit -> text().toDouble());
}

void View::Management::RegistDialog::on_lectiLabel_textChanged()
{
 if ( _lectfLineEdit->text().toInt() > _lectiLineEdit->text().toInt()) {
    int precisionMoney = MAX_MONEY_PRECISION;
    int kw_h = 0, kw_hDisplay = 0;
    double import = 0.00;
    int rangImport;

    kw_h =  _lectfLineEdit->text().toInt() - _lectiLineEdit->text().toInt();
    kw_hDisplay = kw_h;
    _kwhLineEdit->setText(QString::number(kw_hDisplay));

    if (kw_h > 0 && kw_h <= 100)
           rangImport = 1;

       if (kw_h > 100 && kw_h <= 150)
           rangImport = 2;

       if (kw_h > 150 && kw_h <= 200)
           rangImport = 3;

       if (kw_h > 200 && kw_h <= 250)
           rangImport = 4;

       if (kw_h > 250 && kw_h <= 300)
           rangImport = 5;

       if (kw_h > 300 && kw_h <= 350)
           rangImport = 6;

       if (kw_h > 350 && kw_h <= 500)
           rangImport = 7;

       if (kw_h > 500 && kw_h <= 1000)
           rangImport = 8;

       if (kw_h > 1000 && kw_h <= 5000)
           rangImport = 9;

       if (kw_h > 5000)
           rangImport = 10;

       switch (rangImport) {
           case 1:
                   import = (0.09 * kw_h);
                   kw_h = 0;
               break;

           case 2:
                   kw_h -= 100;
                   import = (0.09 * 100) + (0.30 * kw_h);
                   kw_h = 0;
               break;

           case 3:
                   kw_h -= 150;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * kw_h);
                   kw_h = 0;
                break;

           case 4:
                   kw_h -= 200;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * kw_h);
                   kw_h = 0;
               break;

           case 5:
                   kw_h -= 250;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * kw_h);
                   kw_h = 0;
               break;

           case 6:
                    kw_h -= 300;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * kw_h);
                    kw_h = 0;
                break;

           case 7:
                    kw_h -= 350;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * kw_h);
                    kw_h = 0;
                break;

           case 8:
                    kw_h -= 500;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * kw_h);
                    kw_h = 0;
                break;

           case 9:
                    kw_h -= 1000;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * 500) + (03.00 * kw_h);
                    kw_h = 0;
                break;

           case 10:
                    kw_h -= 5000;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * 500) + (03.00 * 4000) + (05.00 * kw_h);
                    kw_h = 0;
                break;
       }


     _importLineEdit->setText(QString::number(import,'f', precisionMoney));
  }
}

void View::Management::RegistDialog::on_lectfLabel_textChanged()
{
  if ( _lectfLineEdit->text().toInt() > _lectiLineEdit->text().toInt()) {
    int precisionMoney = MAX_MONEY_PRECISION;
    int kw_h = 0, kw_hDisplay = 0;
    double import = 0.00;
    int rangImport;

    kw_h =  _lectfLineEdit->text().toInt() - _lectiLineEdit->text().toInt();
    kw_hDisplay = kw_h;
    _kwhLineEdit->setText(QString::number(kw_hDisplay));

    if (kw_h > 0 && kw_h <= 100)
           rangImport = 1;

       if (kw_h > 100 && kw_h <= 150)
           rangImport = 2;

       if (kw_h > 150 && kw_h <= 200)
           rangImport = 3;

       if (kw_h > 200 && kw_h <= 250)
           rangImport = 4;

       if (kw_h > 250 && kw_h <= 300)
           rangImport = 5;

       if (kw_h > 300 && kw_h <= 350)
           rangImport = 6;

       if (kw_h > 350 && kw_h <= 500)
           rangImport = 7;

       if (kw_h > 500 && kw_h <= 1000)
           rangImport = 8;

       if (kw_h > 1000 && kw_h <= 5000)
           rangImport = 9;

       if (kw_h > 5000)
           rangImport = 10;

       switch (rangImport) {
           case 1:
                   import = (0.09 * kw_h);
                   kw_h = 0;
               break;

           case 2:
                   kw_h -= 100;
                   import = (0.09 * 100) + (0.30 * kw_h);
                   kw_h = 0;
               break;

           case 3:
                   kw_h -= 150;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * kw_h);
                   kw_h = 0;
                break;

           case 4:
                   kw_h -= 200;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * kw_h);
                   kw_h = 0;
               break;

           case 5:
                   kw_h -= 250;
                   import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * kw_h);
                   kw_h = 0;
               break;

           case 6:
                    kw_h -= 300;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * kw_h);
                    kw_h = 0;
                break;

           case 7:
                    kw_h -= 350;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * kw_h);
                    kw_h = 0;
                break;

           case 8:
                    kw_h -= 500;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * kw_h);
                    kw_h = 0;
                break;

           case 9:
                    kw_h -= 1000;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * 500) + (03.00 * kw_h);
                    kw_h = 0;
                break;

           case 10:
                    kw_h -= 5000;
                    import = (0.09 * 100) + (0.30 * 50) + (0.40 * 50) + (0.60 * 50) + ( 0.80 * 50) + (01.50 * 50) + (01.80 * 150) + (02.00 * 500) + (03.00 * 4000) + (05.00 * kw_h);
                    kw_h = 0;
                break;
       }


     _importLineEdit->setText(QString::number(import,'f', precisionMoney));
 }
}

bool View::Management::RegistDialog::isSaveable()
{
    return !(_lectiLineEdit -> text().isEmpty()) &&
           !(_lectfLineEdit -> text().isEmpty()) &&
           !(_kwhLineEdit -> text().isEmpty()) &&
           !(_importLineEdit -> text().isEmpty());
}
